#
# Author: SV
# Date: 2015/11/18
# Licence: free
#

fprimeapprox <- function(f, h) {
  return(function(x) { (f(x + h) - f(x)) / h })
}

newton <- function(f, fprime, xi, precision) {
  x <- xi
  while (abs(f(x)) > precision) {
    x <- x - f(x) / fprime(x)
  }
  return(x)
}

g1 <- function(x) {
  return(cos( ((x - 0.5)^2 * (x + 1) +1)/ (x+1) ))  
}

g2 <- function(x) {
  return( sqrt(exp((x-0.25)^2) +1) )
}

main <- function() {
  f1 <- function(x) { -x^3 + x^2 - 2*x+5 }
  f1prime <- function(x) { -3*x^2 + 2*x -2 }
  
  print(newton(f1, f1prime, 0, 1e-6))
  x <- seq(0,2, by=0.001)
  plot(x, f1(x), type = "l")
  abline(h = 0, col = "blue")
  
  f2 <- function(x) { 10*x^4 -20*x^3 - 90*x^2+20*x+80 }
  f2prime <- function(x) { 30*x^3 -60*x^2 - 180*x+20 }
  f2seconde <- function(x) { 90*x^2 -120*x - 180 }

  xmax <- newton(f2prime, f2seconde, 0, 1e-6)
  print(xmax)

  x <- seq(-0.2, 0.5, by=0.0001)
  plot(x, f2(x), type = "l")
  abline(h = f2(xmax), col = "red")  
  
  h <- 0.0001
  f2prime <- fprimeapprox(f2, h)
  f2seconde <- fprimeapprox(f2prime, h)
  
  print(newton(f2prime, f2seconde, 0, 1e-6))
  
  fpart1 <- function(x) { cos(((x-0.5)^2 * (x+1) + 1) / (x+1)) }
  fpart2 <- function(x) { sqrt(exp((x-0.25)^2) + 1) }
  
  fp1 <- fprimeapprox(fpart1, h)
  fpp1 <- fprimeapprox(fp1, h)
  x1 <- newton(f = fp1, fprime = fpp1, xi = 0, precision = 1e-6)
  x2 <- newton(fprimeapprox(fpart2, h), fprimeapprox(fprimeapprox(fpart2, h), h), 0, 1e-6)
  
  print(c(x1, x2))
}