#
# Author: SV
# Date: 2016/11/22
#       2015/11/18
# Licence: free
#
f <- function(x) {
  return( (x[1] - 3)^2 + (x[2] - 1)^2 )
}

gradientf <- function(x) {
  return(c(2 * (x[1] - 3), 2 * (x[2] - 1)))
}

gradientgradientf <- function(x) {
  return(c(2, 2))
}

descenteGradient <- function(x, sigma, precision) {
  v <- c(f(x))
  
  while ( abs(f(x)) > precision) {
    w <- - gradientf(x)
    x <- x + sigma * w
    v <- c(v, f(x))
  }
  
  return(list(x=x, dyn=v))
}

g <- function(x) {
  return(-x^3 + x^2 - 2 * x + 5)
}

gprime <- function(x) {
  return(-3*x^2 + 2*x - 2)
}

ff <- function(x) {
  return(10*x^4-20*x^3-90*x^2+20*x+80)
}

ffprime <- function(x) {
  return(40*x^3-60*x^2-180*x+20)
}

ffseconde <- function(x) {
  return(120*x^2-120*x-180)
}

newton <- function(f, fprime, x, precision) {
  fx <- f(x)
  
  while ( abs(fx) > precision) {
    x <- x - fx / fprime(x)
    fx <- f(x)
  }
  
  return(x)
}

main <- function() {
  cat(f(c(6, 2)), "\n")
  cat(gradientf(c(6, 3)), "\n")
  cat(gradientf(c(3, 1)), "\n")

  res <- descenteGradient(c(6, 3), sigma = 0.001, precision = 1e-6)
  cat("x=", res$x, "f(x)=", f(res$x), "\n")
  plot(log10(res$dyn), type = "l")
  
  x <- seq(0,3, by=0.01)
  plot(x, g(x), type = "l")
  abline(0,0)
  
  xres <- newton(g, gprime, 3, 1e-6)
  cat("x=", xres, "g(x)=", g(xres), "\n")
  
  x <- seq(-1,4, by=0.01)
  plot(x, ff(x), type ="l")
  xres <- newton(ffprime, ffseconde, 2, 1e-6)
  cat("x=", xres, "ff(x)=", ff(xres), "ffprime(x)=", ffprime(xres), "\n")
}

fprimeapprox <- function(f, h) {
  return(function(x) { (f(x + h) - f(x)) / h })
}
